@@ -7,12 +7,24 @@ from json.decoder import JSONDecodeError
 import os
 import math
 import mmh3
-import time 
+import time
 from concurrent.futures import ThreadPoolExecutor
 
-# TODO: these caches dont work in multithreaded env. 
+# TODO: these caches dont work in multithreaded env.
 MSG_CACHE_MAX_LEN = 10000
 
+def is_critical():
+    return str(os.environ.get("OMIGO_CRITICAL", "0")) == "1"
+
+def is_error():
+    return str(os.environ.get("OMIGO_ERROR", "0")) == "1"
+
+def is_warn():
+    return str(os.environ.get("OMIGO_WARN", "0")) == "1"
+
+def is_info():
+    return str(os.environ.get("OMIGO_INFO", "1")) == "1"
+
 def is_debug():
     return str(os.environ.get("OMIGO_DEBUG", "0")) == "1"
 
@@ -58,12 +70,13 @@ def debug_once(msg, msg_cache):
 
          # clear the cache if it has become too big
          if (len(msg_cache) >= MSG_CACHE_MAX_LEN):
-             msg_cache = {} 
+             msg_cache = {}
     else:
         trace(msg)
- 
+
 def info(msg):
-    print("[INFO]: {}".format(msg))
+    if (is_info()):
+        print("[INFO]: {}".format(msg))
 
 def info_once(msg, msg_cache):
     # check if msg is already displayed
@@ -73,12 +86,13 @@ def info_once(msg, msg_cache):
 
          # clear the cache if it has become too big
          if (len(msg_cache) >= MSG_CACHE_MAX_LEN):
-             msg_cache = {} 
+             msg_cache = {}
     else:
         trace(msg)
- 
+
 def error(msg):
-    print("[ERROR]: {}".format(msg))
+    if (is_error()):
+        print("[ERROR]: {}".format(msg))
 
 def error_once(msg, msg_cache):
     # check if msg is already displayed
@@ -88,16 +102,40 @@ def error_once(msg, msg_cache):
 
          # clear the cache if it has become too big
          if (len(msg_cache) >= MSG_CACHE_MAX_LEN):
-             msg_cache = {} 
+             msg_cache = {}
     else:
         trace(msg)
- 
+
+def enable_critical_mode():
+    os.environ["OMIGO_CRITICAL"] = "1"
+
+def enable_error_mode():
+    os.environ["OMIGO_ERROR"] = "1"
+
+def enable_warn_mode():
+    os.environ["OMIGO_WARN"] = "1"
+
+def enable_info_mode():
+    os.environ["OMIGO_INFO"] = "1"
+
 def enable_debug_mode():
     os.environ["OMIGO_DEBUG"] = "1"
 
 def enable_trace_mode():
     os.environ["OMIGO_TRACE"] = "1"
 
+def disable_critical_mode():
+    os.environ["OMIGO_CRITICAL"] = "0"
+
+def disable_error_mode():
+    os.environ["OMIGO_ERROR"] = "0"
+
+def disable_warn_mode():
+    os.environ["OMIGO_WARN"] = "0"
+
+def disable_info_mode():
+    os.environ["OMIGO_INFO"] = "0"
+
 def disable_debug_mode():
     os.environ["OMIGO_DEBUG"] = "0"
 
@@ -105,7 +143,8 @@ def disable_trace_mode():
     os.environ["OMIGO_TRACE"] = "0"
 
 def warn(msg):
-    print("[WARN]: " + msg)
+    if (is_warn() or is_error() or is_critical()):
+        print("[WARN]: " + msg)
 
 def warn_once(msg, msg_cache):
     # check if msg is already displayed
@@ -119,7 +158,7 @@ def warn_once(msg, msg_cache):
 
     else:
         trace(msg)
-  
+
 def is_code_todo_warning():
     return str(os.environ.get("OMIGO_CODE_TODO_WARNING", "0")) == "1"
 
@@ -129,17 +168,20 @@ def print_code_todo_warning(msg):
 
 def url_encode(s):
     if (s is None):
-        return "" 
+        return ""
 
     return urllib.parse.quote_plus(s)
 
 # TODO: this replaces TAB character
 def url_decode(s):
     if (s is None):
-        return "" 
+        return ""
 
     return urllib.parse.unquote_plus(s).replace("\t", " ")
 
+def url_decode_clean(s):
+    return url_decode(s).replace("\n", " ").replace("\v", " ")
+
 # move this to utils
 def parse_encoded_json(s):
     if (s is None):
@@ -186,7 +228,7 @@ def get_counts_map(xs):
             mp[x] = 0
         mp[x] = mp[x] + 1
 
-    return mp 
+    return mp
 
 def report_progress(msg, inherit_message, counter, total):
     report_progress = get_report_progress()
@@ -206,7 +248,6 @@ def merge_arrays(arr_list):
 
     return result
 
-
 def is_array_of_string_values(col_or_cols):
     if (isinstance(col_or_cols, str)):
         return False
@@ -218,47 +259,27 @@ def is_array_of_string_values(col_or_cols):
             break
     return is_array
 
-def is_int_col(xtsv, col):
+def is_numeric(v):
+    return str(v).isnumeric()
+
+def is_float(v):
     try:
-        for v in xtsv.col_as_array(col):
-            if (str(int(v)) != v):
-                return False
+        float(str(v))
+        return True
     except:
         return False
 
-    return True
+def is_int_col(xtsv, col):
+    raise Exception("Deprecated. The to_df() method supports inferring data types")
 
 def is_float_col(xtsv, col):
-    try:
-        xtsv.col_as_float_array(col)
-    except:
-        return False
-
-    return True
+    raise Exception("Deprecated. The to_df() method supports inferring data types")
 
 def is_pure_float_col(xtsv, col):
-    try:
-        found = False
-        for v in xtsv.col_as_float_array(col):
-            if (float(int(v)) != v):
-                found = True
-        if (found == True):
-            return True
-        else:
-            return False
-    except:
-        return False
+    raise Exception("Deprecated. The to_df() method supports inferring data types")
 
 def is_float_with_fraction(xtsv, col):
-    if (is_float_col(xtsv, col) == False):
-        return False
-
-    found = False
-    for v in xtsv.col_as_array(col):
-        if ("." in v):
-            return True
-
-    return False 
+    raise Exception("Deprecated. The to_df() method supports inferring data types")
 
 def compute_hash(x, seed = 0):
     return abs(mmh3.hash64(str(x) + str(seed))[0])
@@ -285,11 +306,11 @@ def run_with_thread_pool(tasks, num_par = 4, wait_sec = 10, post_wait_sec = 0):
             func = task.func
             args = task.args
             kwargs = task.kwargs
-            results.append(func(*args, **kwargs))               
+            results.append(func(*args, **kwargs))
 
         # return
         return results
-    else: 
+    else:
         # start thread pool
         future_results = []
         with ThreadPoolExecutor(max_workers = num_par) as executor:
@@ -329,3 +350,30 @@ def run_with_thread_pool(tasks, num_par = 4, wait_sec = 10, post_wait_sec = 0):
             # return
             return results
 
+def raise_exception_or_warn(msg, ignore_if_missing):
+    # print message if ignore_if_missing flag is true
+    if (ignore_if_missing == True):
+        warn(msg)
+    else:
+        raise Exception(msg)
+
+def strip_spl_white_spaces(v):
+    # check None
+    if (v is None):
+        return None
+
+    # return
+    return str(v).replace("\t", " ").replace("\n", " ").replace("\v", " ")
+
+def resolve_meta_params(xstr, props):
+    # check for None
+    if (xstr is None):
+        return None
+
+    # iterate through properties
+    for k in props.keys():
+        kstr = "{" + k + "}"
+        xstr = xstr.replace(kstr, str(props[k]))
+
+    # return
+    return xstr
